const EOL = require('os').EOL;
const path = require('path');
const fs = require('fs');

/* eslint-env node */
module.exports = {
  description: '',

  normalizeEntityName() {}, // no argument
  
  // locals(options) {
  //   // Return custom template variables here.
  //   return {
  //     foo: options.entity.options.foo
  //   };
  // }
  fileMapTokens(options) {
    return {
      __root__() {
        if (options.inDummy) {
          return 'tests/dummy/app';
        } else {
          return 'app';
        } 
      },
      __mirage__() { return '../mirage'},
      __cloud__() { return '../cloud'}
    }
  },

  afterInstall(options) {
    // Perform extra work here.
    // Add proxy check to config/environment.js 
    let configPath = (options.dummy) ? "tests/dummy/config/environment.js" : "config/environment.js";

    let proxy = "\nfunction usingProxy() {\n"+
                    "\tvar usingProxyArg = !!process.argv.filter(function (arg) {\n"+
                    "\t\treturn arg.indexOf('--proxy') === 0 || arg.indexOf('-pr') === 0 || arg.indexOf('-pxy') === 0;\n"+
                    "\t}).length;\n\n"+
                    "\tvar hasGeneratedProxies = false;\n"+
                    "\tvar proxiesDir = process.env.PWD + '/server/proxies';\n"+
                    "\ttry {\n"+
                    "\t\tfs.lstatSync(proxiesDir);\n"+
                    "\t\thasGeneratedProxies = true;\n"+
                    "\t} catch (e) {}\n\n"+    
                    "\treturn usingProxyArg || hasGeneratedProxies;\n}\n";
    addLineToFile(this, configPath, /'use strict';/, proxy);
    addLineToFile(this, configPath, /when it is created/, "\t\tproxy: usingProxy(),");

    let validatedFormConfig = '\t\t"ember-validated-form": {\n\t\t\tlabel: {\n\t\t\t\tsubmit: "label.save"\n\t\t\t},\n'+
                              '\t\t\tcss: {\n\t\t\t\tgroup: "form-group",\n\t\t\t\tradio: "radio",\n\t\t\t\tcontrol: "form-control",\n'+
                              '\t\t\t\tlabel: "col-form-label",\n\t\t\t\thelp: "small form-text text-danger",'+
                              '\n\t\t\t\thint: "small form-text text-muted",\n\t\t\t\tcheckbox: "checkbox",\n\t\t\t\tbutton: "btn btn-default",'+
                              '\n\t\t\t\tsubmit: "btn btn-primary",\n\t\t\t\tloading: "loading",\n\t\t\t\tvalid: "is-valid",'+
                              '\n\t\t\t\terror: "is-invalid"\n\t\t\t}\n\t\t},\n\t\tgatewayURL: "",\n';
    addLineToFile(this, configPath, /let ENV = {/, validatedFormConfig);

    // Add import of ember_aws_ehipster.css to apps.css
    let stylePath = (options.dummy) ? "tests/dummy/app/styles/app.css" : "app/styles/app.css";
    let importcss = "@import 'ember-aws-ehipster.css';\n";
    let fileContents = importcss + fs.readFileSync(stylePath, 'utf-8');
    fs.writeFileSync(stylePath, fileContents, 'utf-8');

    // Add master path for dealing with entities  
    let routerPath = (options.dummy) ? "tests/dummy/app/router.js" : "app/router.js";
    let routerLine = "\tthis.route('entity-factory', function() {\n\t});\n";
    addLineToFile(this, routerPath, /Router\.map\(function\(\) {/, routerLine);

    // Add special rules to lint config file
    let lintPath = ".template-lintrc.js";
    let lintLines = ",\trules: {\n\t\t\"attribute-indentation\": false,\n\t\t\"block-indentation\": false\n\t}\n";
    addLineToFile(this, lintPath, /extends: 'recommended'/, lintLines);
  }
};

function addLineToFile(ctx, filePath, markerString, addedLine) {
  let fileContents = fs.readFileSync(filePath, 'utf-8');

  if (fileContents.indexOf(addedLine) === -1) {
   //ctx.ui.writeLine("\tAdd line "+addedLine+" to file "+filePath);
   fileContents = fileContents.replace(markerString, [
     '$&',
     addedLine,
   ].join('\n'));
 }
 if (fileContents.indexOf(addedLine) === -1) {
   ctx.ui.writeWarnLine(
     'Unable to update '+filePath+'. You should update this file manually.');
 } else {
   fs.writeFileSync(filePath, fileContents, 'utf-8');
 }
}
